<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * Laskuhari Gateway.
 *
 * Provides a Payment Gateway for Laskuhari invoicing.
 * Edited from WC_Gateway_COD by Datahari Solutions
 *
 * @class 		WC_Gateway_Laskuhari
 * @extends		WC_Payment_Gateway
 * @version		0.9.4
 * @package		WooCommerce/Classes/Payment
 * @author 		WooThemes
 */
class WC_Gateway_Laskuhari extends WC_Payment_Gateway {

    /**
     * Constructor for the gateway.
     */
	public function __construct() {
		$this->id                 = 'laskuhari';
		$this->icon               = apply_filters( 'woocommerce_laskuhari_icon', '' );
		$this->method_title       = __( 'Laskuhari', 'woocommerce' );
		$this->method_description = __( 'Käytä Laskuhari-palvelua tilausten automaattiseen laskuttamiseen', 'woocommerce' );
		$this->has_fields         = false;

		// Load the settings
		$this->init_form_fields();
		$this->init_settings();

		// Get settings
		$this->title              = $this->get_option( 'title' );
		$this->uid                = $this->get_option( 'uid' );
		$this->apikey             = $this->get_option( 'apikey' );
		$this->lahetystapa        = $this->get_option( 'lahetystapa' );
		$this->laskuviesti        = $this->get_option( 'laskuviesti' );
		$this->laskuttaja         = $this->get_option( 'laskuttaja' );
		$this->description        = $this->get_option( 'description' );
		$this->instructions       = $this->get_option( 'instructions', $this->description );
		$this->enable_for_methods = $this->get_option( 'enable_for_methods', array() );
		$this->enable_for_virtual = $this->get_option( 'enable_for_virtual', 'yes' ) === 'yes' ? true : false;

		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
		add_action( 'woocommerce_thankyou_laskuhari', array( $this, 'thankyou_page' ) );

    	// Customer Emails
    	add_action( 'woocommerce_email_before_order_table', array( $this, 'email_instructions' ), 10, 3 );
	}

    /**
     * Initialise Gateway Settings Form Fields.
     */
    public function init_form_fields() {
    	$shipping_methods = array();

    	if ( is_admin() )
	    	foreach ( WC()->shipping()->load_shipping_methods() as $method ) {
		    	$shipping_methods[ $method->id ] = $method->get_title();
	    	}

    	$this->form_fields = array(
			'enabled' => array(
				'title'       => __( 'Käytössä', 'woocommerce' ),
				'label'       => __( 'Ota käyttöön Laskuhari-laskutus', 'woocommerce' ),
				'type'        => 'checkbox',
				'description' => '',
				'default'     => 'no'
			),
			'uid' => array(
				'title'       => __( 'UID', 'woocommerce' ),
				'type'        => 'text',
				'description' => __( 'Laskuhari-tunnuksesi UID (kysy asiakaspalvelusta)', 'woocommerce' ),
				'default'     => __( '', 'woocommerce' ),
				'desc_tip'    => true,
			),
			'apikey' => array(
				'title'       => __( 'API-koodi', 'woocommerce' ),
				'type'        => 'text',
				'description' => __( 'Laskuhari-tunnuksesi API-koodi (kysy asiakaspalvelusta)', 'woocommerce' ),
				'default'     => __( '', 'woocommerce' ),
				'desc_tip'    => true,
			),
			'lahetystapa' => array(
				'title'       => __( 'Lähetystapa', 'woocommerce' ),
				'label'       => __( 'Valitse laskujen lähetystapa', 'woocommerce' ),
				'type'        => 'select',
				'description' => __( 'Valitse tapa, jolla haluat laskuttaa tilaukset', 'woocommerce' ),
				'default'     => 'email',
				'options'     => array(
					'email' => __( 'Sähköpostilasku', 'woocommerce' ),
					'kirje'   => __( 'Kirjelasku', 'woocommerce' ),
					'auto'   => __( 'Kirje, mikäli sähköpostiosoitetta ei annettu', 'woocommerce' ),
					'ei'   => __( 'Tallenna Laskuhariin, älä lähetä', 'woocommerce' )
				)
			),
			'laskuviesti' => array(
				'title'       => __( 'Laskuviesti', 'woocommerce' ),
				'type'        => 'textarea',
				'description' => __( 'Viesti, joka lähetetään saatetekstinä sähköpostilaskun ohessa', 'woocommerce' ),
				'default'     => __( 'Hei,'."\n\n".'Kiitos tilauksestasi. Liitteenä lasku tilaamistasi tuotteista.', 'woocommerce' ),
				'desc_tip'    => true,
			),
			'laskuttaja' => array(
				'title'       => __( 'Laskuttaja', 'woocommerce' ),
				'type'        => 'text',
				'description' => __( 'Laskuttajan nimi, joka näkyy sähköpostilaskun lähettäjänä', 'woocommerce' ),
				'default'     => __( '', 'woocommerce' ),
				'desc_tip'    => true,
			),
			'title' => array(
				'title'       => __( 'Otsikko', 'woocommerce' ),
				'type'        => 'text',
				'description' => __( 'Tämä näkyy maksutavan nimenä asiakkaalle', 'woocommerce' ),
				'default'     => __( 'Laskutus', 'woocommerce' ),
				'desc_tip'    => true,
			),
			'description' => array(
				'title'       => __( 'Kuvaus', 'woocommerce' ),
				'type'        => 'textarea',
				'description' => __( 'Kuvaus, joka näytetään maksutavan yhteydessä', 'woocommerce' ),
				'default'     => __( 'Maksa tilauksesi kätevästi laskulla', 'woocommerce' ),
				'desc_tip'    => true,
			),
			'instructions' => array(
				'title'       => __( 'Ohjeet', 'woocommerce' ),
				'type'        => 'textarea',
				'description' => __( 'Ohjeet, jotka näkyvät tilausvahvistussivulla', 'woocommerce' ),
				'default'     => __( 'Lähetämme sinulle laskun tilauksestasi.', 'woocommerce' ),
				'desc_tip'    => true,
			),
			'enable_for_methods' => array(
				'title'             => __( 'Käytössä näille toimitustavoille', 'woocommerce' ),
				'type'              => 'multiselect',
				'class'             => 'wc-enhanced-select',
				'css'               => 'width: 450px;',
				'default'           => '',
				'description'       => __( 'Jätä tyhjäksi, jos haluat laskutuksen käyttöön kaikille toimitustavoille', 'woocommerce' ),
				'options'           => $shipping_methods,
				'desc_tip'          => true,
				'custom_attributes' => array(
					'data-placeholder' => __( 'Valitse toimitustavat', 'woocommerce' )
				)
			),
			'enable_for_virtual' => array(
				'title'             => __( 'Virtuaalituotteet', 'woocommerce' ),
				'label'             => __( 'Hyväksy laskutus-maksutapa, jos tuote on virtuaalinen', 'woocommerce' ),
				'type'              => 'checkbox',
				'default'           => 'yes'
			)
 	   );
    }

	/**
	 * Check If The Gateway Is Available For Use.
	 *
	 * @return bool
	 */
	public function is_available() {
		$order          = null;
		$needs_shipping = false;

		// Test if shipping is needed first
		if ( WC()->cart && WC()->cart->needs_shipping() ) {
			$needs_shipping = true;
		} elseif ( is_page( wc_get_page_id( 'checkout' ) ) && 0 < get_query_var( 'order-pay' ) ) {
			$order_id = absint( get_query_var( 'order-pay' ) );
			$order    = wc_get_order( $order_id );

			// Test if order needs shipping.
			if ( 0 < sizeof( $order->get_items() ) ) {
				foreach ( $order->get_items() as $item ) {
					$_product = $order->get_product_from_item( $item );
					if ( $_product && $_product->needs_shipping() ) {
						$needs_shipping = true;
						break;
					}
				}
			}
		}

		$needs_shipping = apply_filters( 'woocommerce_cart_needs_shipping', $needs_shipping );

		// Virtual order, with virtual disabled
		if ( ! $this->enable_for_virtual && ! $needs_shipping ) {
			return false;
		}

		// Check methods
		if ( ! empty( $this->enable_for_methods ) && $needs_shipping ) {

			// Only apply if all packages are being shipped via chosen methods, or order is virtual
			$chosen_shipping_methods_session = WC()->session->get( 'chosen_shipping_methods' );

			if ( isset( $chosen_shipping_methods_session ) ) {
				$chosen_shipping_methods = array_unique( $chosen_shipping_methods_session );
			} else {
				$chosen_shipping_methods = array();
			}

			$check_method = false;

			if ( is_object( $order ) ) {
				if ( $order->shipping_method ) {
					$check_method = $order->shipping_method;
				}

			} elseif ( empty( $chosen_shipping_methods ) || sizeof( $chosen_shipping_methods ) > 1 ) {
				$check_method = false;
			} elseif ( sizeof( $chosen_shipping_methods ) == 1 ) {
				$check_method = $chosen_shipping_methods[0];
			}

			if ( ! $check_method ) {
				return false;
			}

			$found = false;

			foreach ( $this->enable_for_methods as $method_id ) {
				if ( strpos( $check_method, $method_id ) === 0 ) {
					$found = true;
					break;
				}
			}

			if ( ! $found ) {
				return false;
			}
		}

		return parent::is_available();
	}


    /**
     * Process the payment and return the result.
     *
     * @param int $order_id
     * @return array
     */
	public function process_payment( $order_id ) {

		$order = wc_get_order( $order_id );

		function laskuhari_curl($post, $url) {
			$ch = curl_init();
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($ch, CURLOPT_URL, $url);
			curl_setopt($ch, CURLOPT_FAILONERROR, 1);
			curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
			curl_setopt($ch, CURLOPT_TIMEOUT, 100);
			curl_setopt($ch, CURLOPT_POST, TRUE);
			curl_setopt($ch, CURLOPT_POSTFIELDS, $post);
			return $ch;
		}
		
		// laskunlähetyksen asetukset
		$laskuhari_uid 			= $this->uid;
		$laskuhari_apikey	 	= $this->apikey;
		$sendername 			= $this->laskuttaja;
		$email_message 			= $this->laskuviesti;

		if(trim(rtrim($email_message)) == "") {
			$email_message = "Liitteenä lasku.";
		}

		// tilaajan tiedot
		$customer 				= $order->get_address('billing');
		$shippingdata 			= $order->get_address('shipping');

		$shipping_different = false;
		foreach ($customer as $key => $cdata) {
			if($shippingdata[$key] != $cdata && isset($shippingdata[$key])) {
				$shipping_different = true;
				break;
			}
		}
		if(!$shipping_different) {
			foreach($shippingdata as $key => $sdata) {
				$shippingdata[$key] = "";
			}
		}

		// tilatut tuotteet
		$products 			= $order->get_items();

		// summat
		$loppusumma 		= $order->get_total();
		$toimitustapa 		= $order->get_shipping_method();
		$toimitusmaksu 		= $order->get_total_shipping();
		$toimitus_vero 		= $order->order_shipping_tax;
		$alennus 			= $order->get_total_discount();
		$cart_discount 		= $order->cart_discount;
		$cart_discount_tax 	= $order->cart_discount_tax;

		// luodaan vahvistuskoodi rajapintaa varten
		$t 				= time();
		$digest_src 	= $laskuhari_uid."+".$t."+".$laskuhari_apikey;
		$cmd 			= "echo -n ".escapeshellarg($digest_src)." | shasum -a 256 -p";
		exec($cmd, $digest, $ret);
		$dt = substr($digest[0],0,64);
		unset($digest);

		// laskunluontirajapinnan URL
		$url 		 =  "https://oma.laskuhari.fi/api/invoice?uid=".$laskuhari_uid."&t=".$t."&dt=".$dt;
		$post 		 =  "action=create";

		$post 		.= 	'&maksuehto='.
						'&pvm='.date('d.m.Y').
						'&lahetetty=0'.
						'&viitteenne='.urlencode("Tilausnro. ".$order->get_order_number()).
						'&yritys='.urlencode($customer['company']).
						'&henkilo='.urlencode($customer['first_name'].' '.$customer['last_name']).
						'&lahiosoite='.urlencode(trim($customer['address_1'].' '.$customer['address_2'])).
						'&postinumero='.urlencode($customer['postcode']).
						'&postitoimipaikka='.urlencode($customer['city']).
						'&toimitus_yritys='.urlencode($shippingdata['company']).
						'&toimitus_henkilo='.urlencode($shippingdata['first_name'].' '.$shippingdata['last_name']).
						'&toimitus_lahiosoite='.urlencode(trim($shippingdata['address_1'].' '.$shippingdata['address_2'])).
						'&toimitus_postinumero='.urlencode($shippingdata['postcode']).
						'&toimitus_postitoimipaikka='.urlencode($shippingdata['city']).
						'&email='.urlencode($customer['email']).
						'&ytunnus='.
						'&puhelin='.urlencode($customer['phone']).
						'&teksti=';

		$i = 1;
		foreach($products as $data) {
			$yks_veroton 			 = $data['line_subtotal']/$data['qty'];
			$yks_verollinen 		 = $yks_veroton + $data['line_subtotal_tax']/$data['qty'];
			$yht_veroton 			 = $data['line_subtotal'];
			$yht_verollinen 		 = $yht_veroton + $data['line_subtotal_tax'];
			$alv 					 = ($yht_verollinen - $yht_veroton) / $yht_veroton * 100;
			$ale 					 = ($yht_veroton - $data['line_total']) / $yht_veroton * 100;
			$ale 					 = 0; //koska pyoristys menee sekaisin laskuharissa...

			$discount_amount 		 = $ale / 100 * $yht_veroton;
			$cart_discount 			-= $discount_amount;
			$cart_discount_tax 		-= $alv / 100 * $discount_amount;

			//kommentoitu koska pyoristys menee sekaisin laskuharissa...
			//$yht_veroton 			 = $data['line_total'];
			//$yht_verollinen 		 = $data['line_total'] + $data['line_tax'];
			
			$post .= 	"&laskurivi[".$i."][tyyppi]=0".
						"&laskurivi[".$i."][koodi]=".
						"&laskurivi[".$i."][nimike]=".urlencode($data['name']).
						"&laskurivi[".$i."][maara]=".urlencode($data['qty']).
						"&laskurivi[".$i."][yks]=".
						"&laskurivi[".$i."][veroton]=".$yks_veroton.
						"&laskurivi[".$i."][alv]=".$alv.
						"&laskurivi[".$i."][verollinen]=".$yks_verollinen.
						"&laskurivi[".$i."][ale]=".$ale.
						"&laskurivi[".$i."][yhtveroton]=".$yht_veroton.
						"&laskurivi[".$i."][yhtverollinen]=".$yht_verollinen.
						"&laskurivi[".$i."][toistovali]=0".
						"&laskurivi[".$i."][ennakko]=0".
						"&laskurivi[".$i."][jatkuvuus]=0".
						"&laskurivi[".$i."][automaatio]=0".
						"&laskurivi[".$i."][toistoale]=0";
			$i++;
		}

		// lisätään toimitusmaksu
		$post .= 	"&laskurivi[".$i."][tyyppi]=0".
					"&laskurivi[".$i."][koodi]=".
					"&laskurivi[".$i."][nimike]=".urlencode("Toimitustapa: ".$toimitustapa).
					"&laskurivi[".$i."][maara]=1".
					"&laskurivi[".$i."][yks]=".
					"&laskurivi[".$i."][veroton]=".($toimitusmaksu).
					"&laskurivi[".$i."][alv]=".round($toimitus_vero/$toimitusmaksu*100, 0).
					"&laskurivi[".$i."][verollinen]=".($toimitusmaksu+$toimitus_vero).
					"&laskurivi[".$i."][ale]=0".
					"&laskurivi[".$i."][yhtveroton]=".$toimitusmaksu.
					"&laskurivi[".$i."][yhtverollinen]=".($toimitusmaksu+$toimitus_vero).
					"&laskurivi[".$i."][toistovali]=0".
					"&laskurivi[".$i."][ennakko]=0".
					"&laskurivi[".$i."][jatkuvuus]=0".
					"&laskurivi[".$i."][automaatio]=0".
					"&laskurivi[".$i."][toistoale]=0";

		$i++;

		// lisätään alennus
		if($cart_discount > 0.009) {
			$post .= 	"&laskurivi[".$i."][tyyppi]=0".
						"&laskurivi[".$i."][koodi]=".
						"&laskurivi[".$i."][nimike]=".urlencode("Alennus").
						"&laskurivi[".$i."][maara]=1".
						"&laskurivi[".$i."][yks]=".
						"&laskurivi[".$i."][veroton]=".($cart_discount*-1).
						"&laskurivi[".$i."][alv]=".round($cart_discount_tax/$cart_discount*100, 0).
						"&laskurivi[".$i."][verollinen]=".(($cart_discount+$cart_discount_tax)*-1).
						"&laskurivi[".$i."][ale]=0".
						"&laskurivi[".$i."][yhtveroton]=".($cart_discount*-1).
						"&laskurivi[".$i."][yhtverollinen]=".(($cart_discount+$cart_discount_tax)*-1).
						"&laskurivi[".$i."][toistovali]=0".
						"&laskurivi[".$i."][ennakko]=0".
						"&laskurivi[".$i."][jatkuvuus]=0".
						"&laskurivi[".$i."][automaatio]=0".
						"&laskurivi[".$i."][toistoale]=0";

			$i++;
		}

		// suoritetaan curl
		$ch = laskuhari_curl($post, $url);
		$response = curl_exec($ch);

		// tarkastetaan virheet
		if(curl_errno($ch)) {
			die("ERROR");
			//curl_errno().curl_error()
		}
		if(stripos(".".$response, "error")) {
			die("ERROR");
			//error
		}

		curl_close($ch);

		$response 			 = json_decode($response, true);
		$laskunro 			 = $response['new_invoice_number'];

		// LASKUN LÄHETYS

		// luodaan POST-pyyntö
		$post = "action=send";
		if($customer['email'] != "") {
			$sendername 	 = $sendername ? $sendername : "Laskutus";
			$post 			.= "&invoiceType=email";
			$post 			.= "&eMailSenderName=".$sendername;
			$post 			.= "&receiverEmail=".urlencode($customer['email']);
			$post 			.= "&eMailSubject=Lasku";
			$post 			.= "&eMailMessage=".urlencode($email_message);
			$kirjelasku 	 = false;
		} else {
			$kirjelasku = true;
			$post .= "&invoiceType=letter";
		}

		if(
			($this->lahetystapa == "kirje" && $kirjelasku == true) ||
			($this->lahetystapa == "email" && $kirjelasku == false) ||
			($this->lahetystapa == "auto")
		) {

			// luodaan vahvistuskoodi rajapintaa varten
			$t 				= time();
			$digest_src 	= $laskuhari_uid."+".$t."+".$laskuhari_apikey;
			$cmd 			= "echo -n ".escapeshellarg($digest_src)." | shasum -a 256 -p";
			exec($cmd, $digest, $ret);
			$dt = substr($digest[0],0,64);
			unset($digest);

			// laskunlähetysrajapinnan URL
			$url 		= "https://oma.laskuhari.fi/api/invoice/".($laskunro*1)."?uid=".$laskuhari_uid."&t=".$t."&dt=".$dt;

			// suoritetaan curl
			$ch 		= laskuhari_curl($post, $url);
			$response 	= curl_exec($ch);

			// tarkastetaan virheet
			if(curl_errno($ch)) {
				die("AERROR");
				//curl_errno().curl_error()
			}
			if(stripos(".".$response, "error")) {
				die("BERROR");
				//error
			}

			curl_close($ch);

			$response = json_decode($response, true);

			$order->update_status( 'pending-payment', __( 'Lasku lähetetty, odottaa maksua', 'woocommerce' ) );

		} else {
			$order->update_status( 'processing', __( 'Odottaa laskutusta', 'woocommerce' ) );
		}


		// Reduce stock levels
		$order->reduce_order_stock();

		// Remove cart
		WC()->cart->empty_cart();

		// Return thankyou redirect
		return array(
			'result' 	=> 'success',
			'redirect'	=> $this->get_return_url( $order )
		);
	}

    /**
     * Output for the order received page.
     */
	public function thankyou_page() {
		if ( $this->instructions ) {
        	echo wpautop( wptexturize( $this->instructions ) );
		}
	}

    /**
     * Add content to the WC emails.
     *
     * @access public
     * @param WC_Order $order
     * @param bool $sent_to_admin
     * @param bool $plain_text
     */
	public function email_instructions( $order, $sent_to_admin, $plain_text = false ) {
		if ( $this->instructions && ! $sent_to_admin && 'laskuhari' === $order->payment_method ) {
			echo wpautop( wptexturize( $this->instructions ) ) . PHP_EOL;
		}
	}
}
